package CaptiveSupply;

/*
 * Program Name: Captive supply effect on the fed-cattle market (Cournot Market)
 * This program used for paper: "THE LONG RUN AND SHORT RUN IMPACT OF CAPTIVE SUPPLIES ON SPOT MARKET PRICE: AN AGENT-BASED ARTIFICIAL MARKET" 
 * 
 * Author: Tong Zhang 
 * email:  zhang.w.tong@gmail.com
 * Date:    07/09/2008
 * updated: 21/Jan/2010
 * */

/* ************************Economic Model Assumptions ****************************************
* 1. Fixed retail market product price = $100;
* 2. Buyers compete product with quantity strategic decisions in spot market; 
*/

/* ******************Program Files description:*************************************************
 * 1. ModelCS: Main program;
 * 2. Buyer: models the buyers' behavior; all buyers have the same behavoral rule but have different initialized starting values;
 * 3. Equilibrium: program to determine the equilibrium;
 * 4. MarketInformationCenter: Storage market information that agents can access and use;  
 * 5. PSO: models the PSO algorithm, have two functions:
 *        a). re-test and selected best locals for each parallel market; 
 *        b). select the best global for all markets;  *     
 * 6. RetailMarket: retail price is perfect elastic, and is $100 all the times;
 * 7. Seller: Seller behavior
 * 8. SpotMarket: Buyers quantity competitive in the spot market;
 *   
 * */

import java.io.FileOutputStream;
import java.io.PrintStream;
import java.text.DecimalFormat;
import java.util.*;

import tools.Print;

public class ModelCS {
	
	/* There are three scenarios to test in the Captive Supply Paper:
	 * 
	 * 1. Test Xia & Sexton's model: packers make fixed number of contracts and fixed the quantity for each contract. 
	 *    which is: FixedContract=true; FixedContractQuantity=true;
	 *    
	 * 2. Relaxed above model as the real world, allow packers make flexible contract number and feeders can adjust their supply quantity of cattle for each contract;
	 *    which is: FixedContract=false; FixedContractQuantity=false; 
	 *    
	 * 3. Test market equilibrium when contact number of each packer is fixed but feeders can adjust their supply quantity
	 *    FixedContract=true; FixedContractQuantity=false; 
	 */
	
	
	/* The boolean value of the following two variables can be changed when each scenario is tested.
	 * The results are stored to the relevant CaptiveSupply folder under eclipse folder tree.
	 */
	
	static boolean FixedContract=false;
	static boolean FixedContractQuantity=false;
		
	static double RetailPrice=100;
	//Assumption End;
	
	static int TotalbuyerNumber=2;// The number will be change to 4 if there are 4 packers in the market
	static int TotalsellerNumber=400;
	
	static double BuyerCapacity=RetailPrice*TotalsellerNumber;//Total cattle number that packer can get if he use price in perfect competition market;
	
	static int EquilibriumNo=100; //Number of runs, results are market price, packers' strategies in the equilibrium for every run time. 
	static int Evolution=500;    //CsRatio Evolutionary
	static int ParallelMarket=20;//The parallel market that PSO algorithm needed 
	static int PSOTestNumber=10; // The 10 history record of packers' strategies,which is used to test the best local strategy.  
	static int WeekPerCycle=4;   
	
	static Random r=new Random(System.currentTimeMillis());

public static void main(String args[]){
	
	double testprint=1;
	
	FileOutputStream out,eout;
	PrintStream printOut,eprintOut;
	//printOut file 
	
	{	try{
		   String name="FixedContract="+FixedContract+". FixedContractQuantity="+FixedContractQuantity+". BuyerNo="+TotalbuyerNumber+".txt";	out=new FileOutputStream(name);	printOut=new PrintStream(out);
	       String ename="Equilibrium. "+" FixedContract="+FixedContract+". FixedContractQuantity="+FixedContractQuantity+". BuyerNo="+TotalbuyerNumber+".txt"; eout=new FileOutputStream(ename); eprintOut=new PrintStream(eout);
	       
	      
	    //Initialization;				
		
		int    i,j,k,pi,fj;
		double tmpDouble;
		int    tmpInt;	
		
		ModelCS	m=new ModelCS();		
		ContractMarket  cm=new ContractMarket(m);
		SpotMarket	    dm=new SpotMarket(m);
		MarketInformationCenter MIC= new MarketInformationCenter(m);
		EquilibriumCriterion em=new EquilibriumCriterion(m,1);
		
		printOut.print("Note:\nMP: Market Price in the spot market;\nCSR:(Captive Supply Ratio)=Number of feeder this packer contract with/total number of feeder; \nCa: capacity (quantity) this packer purchase and process;\n" +
				"" +"\nNumber follow above variables: eg. CSR1 means the captive supply ratio of packer 1" +
						"\n\n");
		
		printOut.print("\nMP\t\t");//market price
		eprintOut.print("MP\t");   //market price
		
		for(i=0;i<m.TotalbuyerNumber;i++){	
		
			if(FixedContract==false)printOut.print("\tCSR"+(i+1));
			printOut.print("\tCa"+(i+1));//Number of cattle this packer purchase and process;
  		    eprintOut.print("CSR"+(i+1)+"\tCa"+(i+1)+"\tPr"+(i+1)+"\t");	
		}


		for(int ec=0;ec<EquilibriumNo;ec++){
		Print.ln(ec);
			
		Buyer[]  b=new Buyer[TotalbuyerNumber];
		Seller[] s=new Seller[TotalsellerNumber];
			
		PSO PSO=new PSO();//Particle swarm optimization algorithm
		
		LinkedList[] sellerList=new LinkedList[m.ParallelMarket];
		LinkedList[] buyerList=new LinkedList[m.ParallelMarket];
		
		Iterator[] sellerit=new Iterator[m.ParallelMarket];
		Iterator[] buyerit=new Iterator[m.ParallelMarket];
		
		LinkedList AvailableBuyerList=new LinkedList();
		
		Seller seller;
		Buyer buyer;
		
		for(i=0;i<TotalbuyerNumber;i++)b[i]=new Buyer(m,i);		
		for(i=0;i<TotalsellerNumber;i++)s[i]=new Seller(m,i);

//Begin Evolution, to find the market equilibrium
//Initialization
           //MIC.ModelInitialization(b, s, m);
//====================================Begin Main Program=======================================//
           for(i=0;i<m.TotalbuyerNumber;i++)b[i].printPR(i);
		
	     long time;
		 em.EquilibriumInitial(); time=System.currentTimeMillis(); 
		 r=new Random(System.currentTimeMillis()*12522782+ec*153654752);
	
		for(int e=-m.PSOTestNumber;e<Evolution;e++){ 
		
		PSO.UpdatePSOParameter(m, e);
		if(e%testprint==0)System.out.print("\n"+e+"");
	
	for(int pm=0;pm<ParallelMarket;pm++){
	
		   AvailableBuyerList.clear();
	
			for(i=0;i<TotalbuyerNumber;i++){ AvailableBuyerList.add(b[i]);b[i].EvolutionInitialization(m, em,PSO, e, pm);} 
			
			for(int week=0;week<WeekPerCycle;week++){		

			for(i=0;i<b.length;i++)b[i].WeekInitialization(m,MIC,em,e,pm);	

			cm.MakeADeal(b, s, m, MIC, week, pm);
			dm.MakeADeal(b, s, m, MIC, week, pm,e);
			dm.PreviousAveragePrice[pm]=dm.AveragePrice[pm];
		    MIC.PreviousAveragePrice[pm]=MIC.AveragePrice[pm];
			for(i=0;i<b.length;i++)	b[i].WeeklyProfit(m, dm,MIC,week, pm);
			
		}//end weeks;

		for(i=0;i<b.length;i++){b[i].StorageParameter(m,pm);b[i].StoragePrevious(m, pm);}
		
	}//end parallel market
	

	for(i=0;i<b.length;i++){
		//First L=10 iterations, best globals are selected from a uniform distribution; 
		if(e<0){b[i].BestGlobalParameter[1]=0.4+0.6*Math.random();b[i].BestGlobalParameter[2]=0.2+0.8*Math.random();}

		else PSO.TestParameterTrade(b[i], b, s, m,cm, dm, MIC,em, PSO, e);
		
		}
	
	if(ec==0&&e>=0)
	{
		printOut.println();
		printOut.print(tools.Print.df1.format(dm.AveragePrice[0])+"\t\t");
		for(i=0;i<m.TotalbuyerNumber;i++) {	b[i].printPriceRatio(m,0, e);	
		
		if(FixedContract==false){if(b[i].PreviousContractRatio[0]<.0001)printOut.print("\t"+0);else printOut.print("\t"+tools.Print.df4.format(b[i].PreviousContractRatio[0]));}
		if(b[i].PreviousCapacityRatio[0]<.0001)printOut.print("\t"+0);else printOut.print("\t"+tools.Print.df4.format(b[i].PreviousCapacityRatio[0]));
		}
			
	}
	//MIC.printResult(m, MIC, b, 0);
	
	//determine market equilibrium 
	
	int parameterNo=2;
	em.EquilibriumStorageParameter(b, m, dm, e, parameterNo);	
	if((e%10==0)&&e>em.CriterionNo)em.TestEquilibrium(b, m, e, parameterNo);//last one: parameter Number
	if(em.JudgeEquilibrium==true||e>=m.Evolution-1){
	time=System.currentTimeMillis()-time;
	em.convergeTime=time;
	em.Storage(b,e);

	//print out the mean of each run; there are 100 runs in total;
	
	
	eprintOut.println();
	eprintOut.print(tools.Print.df1.format(em.AverageMarketPrice[0])+"\t");	
	for(i=0;i<m.TotalbuyerNumber;i++){
	eprintOut.print(tools.Print.df4.format(em.ContractRatio[i][0])+"\t");
	eprintOut.print(tools.Print.df4.format(em.CapacityRatio[i][0])+"\t");
	eprintOut.print(tools.Print.df0.format(em.profit[i][0])+"\t");
	
	}//break;
	}
	
	}//end evolution;	
	
	}//end equilibrium;
	printOut.close();} catch (Exception e){System.err.println ("Error writing to file"); }
		
}

}

}

